/**
 * Circlr - A jQuery plugin for arranging elements in a circle.
 *
 * Options:
 * 'mode' : When not set, the plugin assumes the elements are wider than they 
 * are tall, for example text paragraphs. If set to 1, the items are considered
 * square, and are arranged in a different configuration along the circle's
 * perimeter. See examples.
 *
 * 'item_size' : The height of each element in pixels. Only has a effect in
 * mode 0.
 *
 * 'r' : The radius of the circle. Only has an effect in mode 1.
 *
 * 'ps' : Pixel size, or resolution. Larger values improve performance but
 * position items less accurately.
 *
 * LICENCE
 *
 * Copyright (c) 2012 Erno Kaikkonen, http://blog.ampli.fi/circlr/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *  
 */

(function( $ ) {
  $.fn.circlr = function(options) {
    var settings = $.extend( {
      'item_size' : 80,
      'mode' : 0,
      'r' : 300,
      'ps' : 1
    }, options);
  
    var points = circle_points(this.length, {
      item_size:settings.item_size,
      mode:settings.mode,
      r:settings.r,
      ps:settings.ps
    });
    
    return this.each(function(){
      var a = points.shift();
      var x = a[0];
      var y = a[1];
      $(this).css('position', 'absolute');
      $(this).css('top', y + 'px');
      $(this).css('left', x + 'px');
      $(this).css('display', 'block');
    });

		/**
		 * Returns the points needed for a circle in an array.
		 * 
		 * @param Integer n Number of items
		 * @param Array options The options array.
		 *
		 */
		function circle_points(n, options) {
			var mystical_variable = [0, 0, 0, 3.1, 4.2, 3.4, 4.13, 3.5, 4.1, 3.6];
			var ps = options.ps;
			var item_size = options.item_size;

			if(options.mode == 1) {
				var mode = 1;
				var r = options.r;
				var diff = Math.PI * 2 / n;
			}
			else {
				var mode = 0;
				
				if(n <= 9) {
					m = mystical_variable[n];
				}
				else {
					if(Math.floor(n / 2) == n / 2) {
						m = 4;
					}
					else {
						m = 3.7;
					}
				}
				
				m = m - (item_size / 250);
				
				var r = Math.ceil(n / m * item_size);
				var diff = item_size;
			}

      /*
			console.log('ps: '+ps);
			console.log('mode: '+mode);
			console.log('diff: '+diff);			
			console.log('r: '+r);
			*/
			
			// Arrays for storing the coordinate pairs.
			var used = [];
			var points = [[], [], [], [], [], [], [], []];
			
			var f = 1 - r;
			var dx = 1;
			var dy = -2 * r;
			var x = 0;
			var y = r;

			while(x < y) {
				if(f >= 0) {
					y = y - 1;
					dy += 2;
					f += dy;
				}

				adda((r + x), (r - y), ps, 0, x);				
				adda((r + y), (r + x), ps, 2, x);
				adda((r - x), (r + y), ps, 4, x);
				adda((r - y), (r - x), ps, 6, x);
				adda((r + y), (r - x), ps, 1, r-x);
				adda((r + x), (r + y), ps, 3, r-x);
				adda((r - y), (r + x), ps, 5, r-x);
				adda((r - x), (r - y), ps, 7, r-x);
				
				x = x + 1;
				dx += 2;
				f += dx;
			}				

			var final_points = new Array();
			for(var j = 0;j < 8;j++) {
				for(var i = 0;i < points[j].length ; i++) {
					final_points.push(points[j][i]);
				}			
			}
			
			// Filter out undefined values, and pick only values not on the same y axis.
			var really_final_points = [];
			var prev_angle = -1;
			var prev_x = -1;
			var prev_y = -1;
			for(i = 0;i < final_points.length;i++) {
				if(typeof final_points[i] != 'undefined') {
					if(mode) {
						var angle = get_angle({x:r,y:r}, {x:final_points[i][0],y:final_points[i][1]});
						var new_diff = angle - prev_angle;
						if(prev_angle == -1 || new_diff > diff ){
							really_final_points.push(final_points[i]);
							prev_angle = angle;
							prev_x = final_points[i][0];
							prev_y = final_points[i][1];
						}
					}
					else {
						if(Math.abs(final_points[i][1] - prev_y) >= diff || prev_y == -1) {
							really_final_points.push(final_points[i]);
							prev_x = final_points[i][0];
							prev_y = final_points[i][1];
						}
						else if(final_points[i][1] == prev_y) {
							if(final_points[i][0] < prev_x) {
								really_final_points.pop();
								really_final_points.push(final_points[i]);
							}								
							prev_x = final_points[i][0];
							prev_y = final_points[i][1];
						}	
						
					}
				}
			}

			return really_final_points;	
			
			/**
			 * Add a coordinate pair to the points array, if it's not there yet.
			 *
			 * The points array starts off as a two-dimensional array, to allow
			 * us to sort the points along the circle arc even though we do eight
			 * segments at a time.
			 *
			 */
			function adda(tx, ty, ps, i1, i2) {
				tx = tx * ps;
				ty = ty * ps;
				if($.inArray(tx + ':' + ty, used) == -1) {
					points[i1][i2] = [tx, ty];
					used.push(tx + ':' + ty);
				}
			}
		
		  /**
		   * Function from http://beradrian.wordpress.com/2009/03/23/calculating-the-angle-between-two-points-on-a-circle/
		   *
		   */
			function get_angle(center, p1) {
				var p0 = {x: center.x, y: center.y - Math.sqrt(Math.abs(p1.x - center.x) * Math.abs(p1.x - center.x) + Math.abs(p1.y - center.y) * Math.abs(p1.y - center.y))};
				var ret = (2 * Math.atan2(p1.y - p0.y, p1.x - p0.x));// * 180 / Math.PI;
				
				return ret;
			}
		}
  };
})( jQuery );
